program exercise24
  implicit none

  ! newton法でf(x)=0の解を求めるプログラム
  
  real*8 :: x_l, x_r, x_new, f_new, eps=1.0d-14
  integer :: iter

  ! xの境界値
  x_l = 0.0d0
  x_r = 10.0d0
  
  ! 反復の回数カウンタ
  iter = 1

  ! xの初期値
  x_new = (x_l + x_r)/2.0d0
  
  ! 反復のループ
  do 

     ! newton法で次の点を計算
     x_new = x_new - f(x_new)/dfdx(x_new)
     f_new = f(x_new)

     ! 反復の途中経過を出力
     print *, "iter = ", iter, "x_new = ", x_new, "f_new = ", f_new

     ! 境界値より外に出たら計算を終了する
     if(x_new .lt. x_l .or. x_new .gt. x_r) then
        write(0,*) "x_new out of range"
        stop
     end if
         
     ! f_newが十分ゼロに近づいたらdoループから抜ける
     if( abs(f_new) .lt. eps) exit

     ! 反復カウンタを１増やす
     iter = iter + 1

     ! もし問題が起きた場合1000回で反復を強制的に終了する
     if( iter .eq. 1000) stop "iteration did not converge"
     
     
  end do

  
contains

  ! f(x)の値を返す関数副プログラム
  function f(x)
    real*8, intent(in) :: x
    real*8 :: f
    f = sin(x/4.0d0) - cos(x/4.0d0)

    return

  end function f

  ! df/dxの値を返す関数副プログラム
  function dfdx(x)
    real*8, intent(in) :: x
    real*8 :: dfdx

    dfdx = (sin(x/4.0d0) + cos(x/4.0d0)) / 4.0d0
    return
  end function dfdx
  
end program exercise24
