program exercise25
  implicit none

  ! 多次元newton法でf(x,y)=0, g(x,y)=0の解を求めるプログラム
  
  real*8 :: x_new(1:2), f_new(1:2), det, eps=1.0d-14
  integer :: iter

  real*8 :: jacob(1:2,1:2), jinv(1:2,1:2)
  
  ! 反復の回数カウンタ
  iter = 1

  ! xの初期値
  x_new(1) = 1.0d0
  x_new(2) = 1.0d0
  f_new = f(x_new(1:2))
  
  ! 反復のループ
  do 

     ! jacobian
     jacob = dfdx(x_new(1:2))

     ! jacobianの逆行列
     det = jacob(1,1)*jacob(2,2) - jacob(2,1)*jacob(1,2)
     jinv(1,1) = jacob(2,2)
     jinv(2,2) = jacob(1,1)
     jinv(1,2) = -jacob(1,2)
     jinv(2,1) = -jacob(2,1)
     jinv(1:2,1:2) = det**(-1) * jinv(1:2,1:2)
     
     ! newton法で次の点を計算
     
     x_new(1:2) = x_new(1:2) - matmul( jinv(1:2,1:2), f_new(1:2)) 
     f_new = f(x_new(1:2))
     
     ! 反復の途中経過を出力
     print *, "iter = ", iter
     print *, "x_new = ", x_new(1), x_new(2)
     print *, "f_new = ", f_new(1), f_new(2)

     ! f_newが十分ゼロに近づいたらdoループから抜ける
     if( sqrt(f_new(1)**2 + f_new(2)**2) .lt. eps) exit

     ! 反復カウンタを１増やす
     iter = iter + 1

     ! もし問題が起きた場合1000回で反復を強制的に終了する
     if( iter .eq. 1000) stop "iteration did not converge."
     
  end do
  
contains

  ! f(x,y)の値を返す関数副プログラム
  function f(x)
    real*8, intent(in) :: x(1:2)
    real*8 :: f(1:2)
    f(1) = x(1)**2 + x(2)**2 - 10.0d0
    f(2) = x(1) - x(2)**3
    return
  end function f

  ! df/dxの値を返す関数副プログラム
  function dfdx(x)
    real*8, intent(in) :: x(1:2)
    real*8 :: dfdx(1:2,1:2)

    dfdx(1,1) = 2.0d0 * x(1)
    dfdx(1,2) = 2.0d0 * x(2)

    dfdx(2,1) =  1.0d0
    dfdx(2,2) = -3.0d0 * x(2)**2
    
    return
  end function dfdx

end program exercise25
