program exercise26
  implicit none

  ! broyden法でf(x,y)=0, g(x,y)=0の解を求めるプログラム
  
  real*8 :: x_bro(1:2), f_bro(1:2), f_bro2(1:2), deltax_bro(1:2)
  integer :: iter, i,j

  real*8 :: eps=1.0d-14
  
  real*8 :: binv(1:2,1:2), denom, nom1(1:2), nom2(1:2), deltaf_bro(1:2)
  
  ! 反復の回数カウンタ
  iter = 1

  ! xの初期値
  x_bro(1) = 0.0d0
  x_bro(2) = 0.0d0
  f_bro = f(x_bro(1:2)) 

  print *, "initial x   =", x_bro
  print *, "initial f(x)=", f_bro
  
  ! jacobianの逆行列の初期値(単位行列に設定)
  binv(1:2,1:2) = 0.0d0
  binv(1,1) = 1.0d0
  binv(2,2) = 1.0d0

  ! 反復のループ
  do 

     ! newton法で次の点を計算
     ! \delta x^{(k)}
     deltax_bro(1:2) = -matmul( binv(1:2,1:2), f_bro(1:2))

     ! x^{(k+1)}
     x_bro(1:2) = x_bro(1:2) + deltax_bro(1:2)

     ! f(x^{(k+1)})
     f_bro2 = f(x_bro(1:2))

     ! \delta f
     deltaf_bro(1:2) = f_bro2(1:2) - f_bro(1:2)

     ! f_broにf(x^{(k+1)})を代入して次の反復計算で用いる
     f_bro = f_bro2

     ! b^{-1}行列の分母部分
     denom = dot_product( deltax_bro(1:2), matmul( binv(1:2,1:2), deltaf_bro(1:2)))

     ! b^{-1}行列の分子部分(左側)
     nom1(1:2) = deltax_bro(1:2) - matmul( binv(1:2,1:2), deltaf_bro(1:2))
     ! b^{-1}行列の分子部分(右側)
     nom2(1:2) = matmul(deltax_bro(1:2), binv(1:2,1:2))

     ! b^{(k+1)}^{-1}を計算
     do i = 1, 2
     do j = 1, 2
        binv(i,j) = binv(i,j) + nom1(i)*nom2(j)/denom
     end do
     end do
   
     ! 反復の途中経過を出力
     print *, "iter = ", iter
     print *, "x    = ", x_bro(1), x_bro(2)
     print *, "f(x) = ", f_bro(1), f_bro(2)

     ! f_broが十分ゼロに近づいたらdoループから抜ける
     if( abs(sqrt(f_bro(1)**2 + f_bro(2)**2)) .lt. eps) exit

     ! 反復カウンタを１増やす
     iter = iter + 1

     ! もし問題が起きた場合1000回で反復を強制的に終了する
     if( iter .eq. 1000) stop "iteration did not converge."
     
  end do

  
contains

  ! f(x,y)の値を返す関数副プログラム
  function f(x)
    real*8, intent(in) :: x(1:2)
    real*8 :: f(1:2)
    f(1) = x(1)**2 + x(2)**2 - 10.0d0
    f(2) = x(1) - x(2)**3
    return
  end function f

end program exercise26
